package com.composition.android.ad.gromore.union

import android.content.Context
import android.content.Intent
import android.os.SystemClock
import android.view.ViewGroup
import androidx.core.view.isVisible
import com.bytedance.sdk.openadsdk.AdSlot
import com.bytedance.sdk.openadsdk.mediation.MediationConstant
import com.bytedance.sdk.openadsdk.mediation.bridge.custom.splash.MediationCustomSplashLoader
import com.bytedance.sdk.openadsdk.mediation.custom.MediationCustomServiceConfig
import com.composition.android.lib.ad.util.adLogWithFrame
import com.qq.e.ads.splash.SplashAD
import com.qq.e.ads.splash.SplashADListener
import com.qq.e.comm.pi.IBidding
import com.qq.e.comm.util.AdError
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext


/**
 * @classname: GroMore 自定义广告 --- 优量汇ADN  开屏广告
 * @author: LiuXiaoLong
 * @date: 2023/9/18
 * description:
 */
class CustomUnionAppOpenRequestImpl : MediationCustomSplashLoader() {

    private var splashAd: SplashAD? = null

    /**
     * 为防止无广告时造成视觉上类似于"闪退"的情况，设定无广告时页面跳转根据需要延迟一定时间，demo
     * 给出的延时逻辑是从拉取广告开始算开屏最少持续多久，仅供参考，开发者可自定义延时逻辑，如果开发者采用demo
     * 中给出的延时逻辑，也建议开发者考虑自定义minSplashTimeWhenNoAD的值（单位ms）
     */
    private val minSplashTimeWhenNoAD = 2000

    /**
     * 记录拉取广告的时间
     */
    private var fetchSplashADTime: Long = 0

    private var expireTimestamp: Long = 0L


    override fun load(p0: Context?, p1: AdSlot?, p2: MediationCustomServiceConfig?) {
        GlobalScope.launch(Dispatchers.IO) {
            adLogWithFrame(
                msgs = arrayOf(
                    "加载GroMore自定义广告-开屏",
                    "广告商:优量汇(Union)",
                    "广告ID:${p2?.adnNetworkSlotId}"
                )
            )
            splashAd = SplashAD(p0, p2?.adnNetworkSlotId, object : SplashADListener {
                override fun onADDismissed() {
                    adLogWithFrame(msg = "GroMore-开屏-自定义优量汇(Union), 回调onADDismissed")
                    callSplashAdDismiss()
                }

                override fun onNoAD(p0: AdError?) {
                    adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union),加载失败-ErrorCode:${p0?.errorCode}- ${p0?.errorMsg}"))
                    /**
                     * 为防止无广告时造成视觉上类似于"闪退"的情况，设定无广告时页面跳转根据需要延迟一定时间，demo
                     * 给出的延时逻辑是从拉取广告开始算开屏最少持续多久，仅供参考，开发者可自定义延时逻辑，如果开发者采用demo
                     * 中给出的延时逻辑，也建议开发者考虑自定义minSplashTimeWhenNoAD的值
                     **/
                    val alreadyDelayMills: Long = System.currentTimeMillis() - fetchSplashADTime //从拉广告开始到onNoAD已经消耗了多少时间
                    val shouldDelayMills: Long = if (alreadyDelayMills > minSplashTimeWhenNoAD) 0 else (minSplashTimeWhenNoAD
                            - alreadyDelayMills) //为防止加载广告失败后立刻跳离开屏可能造成的视觉上类似于"闪退"的情况，根据设置的minSplashTimeWhenNoAD
                    GlobalScope.launch(Dispatchers.IO) {
                        adLogWithFrame(msg = "延迟:${shouldDelayMills}ms后退出")
                        delay(shouldDelayMills)
                        callSplashAdSkip()
                    }
                }

                override fun onADPresent() {
                    adLogWithFrame(msg = "GroMore-开屏-自定义优量汇(Union), 回调onADPresent")
                    callSplashAdShow()
                }

                override fun onADClicked() {
                    adLogWithFrame(msg = "GroMore-开屏-自定义优量汇(Union), 回调onADClicked")

                    callSplashAdClicked()
                }

                override fun onADTick(p0: Long) {
                    adLogWithFrame(msg = "GroMore-开屏-自定义优量汇(Union), 回调倒计时:${p0}")
                }

                override fun onADExposure() {

                }

                override fun onADLoaded(expireTimestamp: Long) {
                    this@CustomUnionAppOpenRequestImpl.expireTimestamp = expireTimestamp
                    adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union),加载成功"))
                    splashAd?.let {
                        if (biddingType == MediationConstant.AD_TYPE_CLIENT_BIDING) {
                            var epcm = splashAd?.ecpm?.toDouble()
                            epcm?.apply {
                                adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union), 客户端竞价，ecpm:${this}"))
                                if (this < 0) {
                                    epcm = 0.0
                                }
                                callLoadSuccess(epcm!!)
                            }
                        } else {
                            adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union), 竞价方式:${biddingType}"))
                            callLoadSuccess()
                        }
                    }
                }
            })
            fetchSplashADTime = System.currentTimeMillis();
            splashAd?.fetchAdOnly()
        }
    }

    override fun showAd(viewGroup: ViewGroup?) {
        adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union), 展示开屏广告"))
        GlobalScope.launch(Dispatchers.Main) {
            delay(400)
            splashAd?.showAd(viewGroup)
        }
    }

    override fun isReadyCondition(): MediationConstant.AdIsReadyStatus {
        if (splashAd == null) {
            adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union), splashAd为空， 还未准备好"))
            return MediationConstant.AdIsReadyStatus.AD_IS_NOT_READY
        }
        if (expireTimestamp != 0L && SystemClock.elapsedRealtime() > expireTimestamp) {
            adLogWithFrame(msgs = arrayOf("GroMore-开屏-自定义优量汇(Union), 广告貌似已经无效了，过期时间:${expireTimestamp}"))
            return MediationConstant.AdIsReadyStatus.AD_IS_EXPIRED
        }
        return if (splashAd?.isValid == true) MediationConstant.AdIsReadyStatus.AD_IS_READY else MediationConstant.AdIsReadyStatus.AD_IS_NOT_READY
    }

    override fun receiveBidResult(win: Boolean, winnerPrice: Double, loseReason: Int, extra: MutableMap<String, Any>?) {
        super.receiveBidResult(win, winnerPrice, loseReason, extra)
        val reason = when (loseReason) {
            1 -> "价格较低"
            2 -> "广告返回超时"
            10001 -> "其他原因"
            -1 -> "胜出"
            else -> {
                ""
            }
        }
        adLogWithFrame(
            msgs = arrayOf(
                "GroMore-开屏-自定义优量汇(Union)",
                "竞价结果:${if (win) "竞价胜利" else "竞价输了"}",
                "竞价胜利价格:${winnerPrice}",
                "失败原因:${reason}",
                "extra: $extra"
            )
        )
        if (win) {
            if (winnerPrice > 0) {
                splashAd?.sendWinNotification(
                    mapOf(
                        IBidding.EXPECT_COST_PRICE to winnerPrice
                    )
                )
            }
        } else {
            if (loseReason == 1) {
                splashAd?.sendLossNotification(
                    mapOf(
                        IBidding.LOSS_REASON to 2 // 其他广告出价更高
                    )
                )
            }
        }
    }
}